/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

interface nsIDOMMozSmsMessage;

[scriptable, builtinclass, uuid(f12f9208-0abd-4d95-9a27-93a74b5e47a2)]
interface nsIDOMMozSmsEvent : nsIDOMEvent
{
  readonly attribute nsIDOMMozSmsMessage message;

  [noscript] void initMozSmsEvent(in DOMString aType,
                                  in boolean aCanBubble,
                                  in boolean aCancelable,
                                  in nsIDOMMozSmsMessage aMessage);
};
