/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef _NECKO_CONFIG_H_
#define _NECKO_CONFIG_H_

#define NECKO_COOKIES 1

#define NECKO_WIFI 1

#define NECKO_PROTOCOL_about 1
#define NECKO_PROTOCOL_data 1
#define NECKO_PROTOCOL_device 1
#define NECKO_PROTOCOL_file 1
#define NECKO_PROTOCOL_ftp 1
#define NECKO_PROTOCOL_http 1
#define NECKO_PROTOCOL_res 1
/* #undef NECKO_PROTOCOL_rtsp */
#define NECKO_PROTOCOL_viewsource 1
#define NECKO_PROTOCOL_websocket 1
#define NECKO_PROTOCOL_wyciwyg 1

#endif
